<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>{{page.meta.title}}</title>
<meta name="viewport" content="width=device-width, initial-scale=1" />
<meta http-equiv="refresh" content="0; URL={{url_for(page)}}" />
</head>
<body>
  <a href="{{url_for(page)}}">The page has moved, you should be redirected automatically.</a>
</body>
</html>
