{% extends "base.html" %}

{% block content %}
<h1>Archive for tag {{tag.name}}</h1>

<ul>
{% for p in pages %}
<li><a href="{{url_for(p)}}">{{p.meta.title}}</a></li>
{% endfor %}
</ul>

{% endblock %}

