{% extends "base.html" %}
{% block content %}
  <div class="page-header">
    <h1>{{page.meta.title}}</h1>
  </div>

  {{content|safe}}

  <p>
  {% for tag in tags %}
  <a href="{{ url_for_tag(tag) }}" class="label label-default">{{tag}}</a>
  {% endfor %}
  </p>

  <p><small>{{page.meta.date}}</small></p>
{% endblock %}
