<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <!-- The above 3 meta tags *must* come first in the head; any other head content must come *after* these tags -->
    <title>{% block title %}{{title}}{% endblock %}</title>

    <!-- Bootstrap -->
    <link href="{{ url_for('/css/bootstrap.min.css') }}" rel="stylesheet">
    <link href="{{ url_for('/css/site.css') }}" rel="stylesheet">
    <link href="{{ url_for('/css/github.css') }}" rel="stylesheet">
    {% block favicon %}<link rel="icon" href="{{ url_for('images/favicon.ico') }}">{% endblock %}
    {% block head %}
    {% endblock %}
  </head>
  <body>
    <!-- jQuery (necessary for Bootstrap's JavaScript plugins) -->
    <script src="{{ url_for('/js/jquery.min.js') }}"></script>
    <!-- Include all compiled plugins (below), or include individual files as needed -->
    <script src="{{ url_for('/js/bootstrap.min.js') }}"></script>

    <!-- Fixed navbar -->
    <nav class="navbar navbar-inverse navbar-fixed-top">
      <div class="container">
        <div class="navbar-header">
          <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar" aria-expanded="false" aria-controls="navbar">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
          </button>
          <a class="navbar-brand" href="{{SITE_ROOT}}">{{SITE_NAME}}</a>
        </div>
        <div id="navbar" class="navbar-collapse collapse">
          <ul class="nav navbar-nav">
            {% if has_page('/blog') %}
            <li><a href="{{ url_for('/blog') }}">Blog</a></li>
            {% endif %}
            {% for taxonomy in taxonomies() %}
            <li><a href="{{ url_for(taxonomy) }}">Tags</a></li>
            {% endfor %}
            <!--
            <li class="active"><a href="#">Home</a></li>
            <li><a href="#about">About</a></li>
            <li><a href="#contact">Contact</a></li>
            <li class="dropdown">
              <a href="#" class="dropdown-toggle" data-toggle="dropdown" role="button" aria-haspopup="true" aria-expanded="false">Dropdown <span class="caret"></span></a>
              <ul class="dropdown-menu">
                <li><a href="#">Action</a></li>
                <li><a href="#">Another action</a></li>
                <li><a href="#">Something else here</a></li>
                <li role="separator" class="divider"></li>
                <li class="dropdown-header">Nav header</li>
                <li><a href="#">Separated link</a></li>
                <li><a href="#">One more separated link</a></li>
              </ul>
            </li>
            -->
          </ul>
        </div><!--/.nav-collapse -->
      </div>
    </nav>

    <div class="container" role="main">
    {% block content %}
    {% endblock %}
    </div>

    {% block footer %}
    <div class="panel panel-default">
      <div class="panel-footer">
        Generated with <a href="https://github.com/spanezz/staticsite">staticsite</a> on {{now}}.
      </div>
    </div>
    {% endblock %}
  </body>
</html>
