{% extends "base.html" %}
{% block content %}

<h1>{{SITE_NAME}}</h1>

<p>This is the content of the home page for the site.</p>

<p>You do not need to write it as Jinja2+HTML, it can be <tt>index.md</tt>
instead, depending on your needs.</p>

<p>With a Jinja2 template, you can do interesting things like this:</p>

<h2>Latest <a href="{{ url_for('/blog/index.html') }}">blog posts</a></h2>

<ul>
  {% for page in site_pages(path="blog/*", limit=10) %}
  <li><a href="{{ url_for(page) }}">{{page.meta.date}}: {{page.meta.title}}</a></li>
  {% endfor %}
</ul>

{% endblock %}
