# Name of the website
SITE_NAME = "Example web site"

# Author of the website
SITE_AUTHOR = "Example author"

# Absolute URL to the root of the website
SITE_URL = "https://www.example.org"

# Time zone used for site posts
TIMEZONE = "Europe/Rome"

# Root directory of the site in the URLs we generate.
# If you are publishing the site at /prefix instead of root of the domain,
# override this with /prefix
# SITE_ROOT = "/"

# Directory with the source content of the site
# CONTENT = "content"

# Theme used to render the content
# Default: the one installed in the system
THEME = "theme"

# Directory with "archetypes" (templates used by ssite new)
#ARCHETYPES = "archetypes"

# Directory where the static site will be written by build
#OUTPUT = "web"
